
#include "sssd.h"
#include "internals.h"

/*
   Stop sssd system: turn off all playing sounds, kill thread, close
   device.  Returns -1 on error.
*/

int sssd_stopAudio( struct sssdaudio* ga )
{

#ifndef __NOPRINTING__
   puts("sssd_stopAudio()");
#endif

    if( ga->audio_fd == -1 ) {
        ga->lastERR=ERR_NOT_INITIALIZED;
        return( ERR_NOT_INITIALIZED );
    }
    
    ga->lastERR = 0;

#ifndef __NOPRINTING__
   puts("sssd_stopAudio: stopBackground");
#endif
    sssd_stopBackground(ga);
    
#ifndef __NOPRINTING__
   puts("sssd_stopAudio: stop mixer thread");
#endif
    /* Stop grabbing thread */
    LOCK_BUFFERS_NP(ga);
    ga->threadSignal=1;
    while(ga->threadSignal!=2) {
       UNLOCK_BUFFERS(ga);
       usleep(100000);
       LOCK_BUFFERS(ga);
    }
    usleep(100000);
    UNLOCK_BUFFERS(ga);

#ifndef __NOPRINTING__
   puts("sssd_stopAudio: free samples");
#endif
    sssd_freeSamples(ga);

    /* Close device */
#ifndef __NOPRINTING__
   puts("sssd_stopAudio: close device");
#endif
    close(ga->audio_fd);
    ga->audio_fd = -1;


#ifndef __NOPRINTING__
   puts("sssd_stopAudio: free playbuffer");
#endif
    /* free play buffer */
    if(ga->playBuffer!=NULL) {
       free(ga->playBuffer);
       ga->playBuffer=NULL;
    }

#ifndef __NOPRINTING__
   puts("sssd_stopAudio: free sample structs");
#endif
    /* Free Sample structs */
    if(ga->samples!=NULL) {
       free(ga->samples);
       ga->samples=NULL;
    }

#ifndef __NOPRINTING__
   puts("sssd_stopAudio: free playing structs");
#endif
    /* Free Playing structs */
    if(ga->playing!=NULL) {
       free(ga->playing); 
       ga->playing=NULL;
    }
    
    
#ifndef __NOPRINTING__
   puts("sssd_stopAudio: destroy mutex");
#endif
    /* destroy mutex */
   DELETE_MUTEX(ga);   
    
   return( ga->lastERR );
}
