
#ifndef __INTERNALS_H
#define __INTERNALS_H

#define MIN(a,b)    (((a) < (b)) ? (a) : (b))
#define MAX(a,b)    (((a) > (b)) ? (a) : (b))

struct ga_sample
{
    void* data;
    void* dataendmarker;
    int numsamples;
    int flags;
};

/* FLAG VALUES

   bit 01:  0=Cached sample/1=dynamic sample

*/
#define gaIS_DYNAMIC   0x00000001
#define gaCHAN_CONVERT 0x00000002 /* stereo sample in mono mixing thread or mono sample in stereo mixing thread */

struct ga_playing
{
    int sample_id;
    int playing_id;
    void* rootpos;
    void* curpos;
    void* endpos;

    void* playpos;
    void* playend;
    void* looppos;
    void* loopend;

    int a,b;
    int repeat_count;

    int priorityQueue;
    int voiceflags;
};


/* Voice Flags */
  #define PLAYFLAG_CHANCHANGE 0x0002  /* stereo sample in mono mixing thread or mono sample in stereo mixing thread */


  #define BACKGROUND_MOD_PLAY_CMD "xmp"
  #define BACKGROUND_OGG_PLAY_CMD "oggdec"


#ifndef MAX_AUDIO_BUFFERS
#define MAX_AUDIO_BUFFERS 16
#endif

#ifdef __LINUX__

  #include <stdlib.h>
  #include <string.h>
  #include <unistd.h>
  #include <fcntl.h>
  #include <sys/stat.h>
  #include <sys/ioctl.h>
  #include <sys/soundcard.h>
  #include <memory.h>
  #include <sys/time.h>
  #ifdef _INCLUDE_ESD
    #include <sys/types.h>
    #include <sys/socket.h>
  #endif

  #define HAVE_REGISTER 1
  #define INLINE_PROTO inline

  #define sample16to8(a) (unsigned char) ((a>>8)+0x80);
  #define sample8to16(a) (short)(((int)(a)-0x80)<<8)

  /* Locks/Unlocks the buffer mutex */
  #define LOCK_BUFFERS(ga)    pthread_mutex_lock(&ga->buffer_mutex)
  #define UNLOCK_BUFFERS(ga)  pthread_mutex_unlock(&ga->buffer_mutex)
  #define DELETE_MUTEX(ga)    pthread_mutex_destroy(&ga->buffer_mutex)
  #define EXIT_THREAD(ga)     pthread_exit(0)

  /* Locks/Unlocks the buffer mutex if not paused */
  #define LOCK_BUFFERS_NP(ga)    if (!audioPaused(ga)) LOCK_BUFFERS(ga)
  #define UNLOCK_BUFFERS_NP(ga)  if (!audioPaused(ga)) UNLOCK_BUFFERS(ga)

  /* play the audio buffer- dummy arg needed by some compilers */
  #define SEND_BUFFER_TO_DEVICE(a) write(ga->audio_fd, a, abufsize )

  /* what makes us sleep */
  #define SLEEP_TYPE unsigned long long
  #define SLEEP_CALL(a) usleep(a)

   /* Default OSS device to use */
  #define DEFAULT_DEVICE_NAME "/dev/dsp"

  #define DEFAULT_NUM_AUDIO_BUFFERS 4

#endif  /* Linux */


#ifdef _WIN32_WCE

  #if _MSC_VER > 1000
  #pragma once
  #endif // _MSC_VER > 1000

  #define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
  #include <windows.h>
  #include <winbase.h>
  #include <mmsystem.h>

  #define HAVE_REGISTER 1
  #define INLINE_PROTO

  //#define WIN_DEBUG  // output debugging info from windows specific routines

  #define sample16to8(a) (unsigned char) ((a>>8)+0x80);
  #define sample8to16(a) (short)(((int)(a)-0x80)<<8)

  /* Locks/Unlocks the buffer mutex */
  #define LOCK_BUFFERS(ga)    WaitForSingleObject(&ga->buffer_mutex, INFINITE)
  #define UNLOCK_BUFFERS(ga)  ReleaseMutex(&ga->buffer_mutex)
  #define EXIT_THREAD(ga)     ExitThread(0)

  /* Locks/Unlocks the buffer mutex if not paused */
  #define LOCK_BUFFERS_NP(ga)    if (!audioPaused(ga)) LOCK_BUFFERS(ga)
  #define UNLOCK_BUFFERS_NP(ga)  if (!audioPaused(ga)) UNLOCK_BUFFERS(ga)

  /* play the audio buffer- dummy arg needed by some compilers */
  #define SEND_BUFFER_TO_DEVICE(a) WIN_AudioWrite(ga,a, abufsize)

  /* what makes us sleep */
  #define SLEEP_TYPE long
  #define SLEEP_CALL(a) Sleep(a>>10)

  /* Misc stuff missing from Windows */
  #ifndef FILENAME_MAX
     #define FILENAME_MAX 256
  #endif

  #define DEFAULT_NUM_AUDIO_BUFFERS 4

   struct WinPlatformData {
	   int buffCount;
	   int nextBuff;
	   int sleepMS;
	   WAVEHDR  *headers;
	   void **   WinBuffer;
	   CRITICAL_SECTION criticalSection;
	   HWAVEOUT hWaveOut;
   };


  /* Until we fix background */
#define popen(a,b) a=a
#define pclose(a) a=a
#define fread(a,b,c,d) 0

#endif  /* WinCE */


#ifdef __ELATE__

  #include <stdlib.h>
  #include <string.h>
  #include <unistd.h>
  #include <elate/kn.h>
  #include <elate/devices.h>

  #define HAVE_REGISTER 1
  #define INLINE_PROTO inline

  #define sample16to8(a) (unsigned char) ((a>>8)+0x80);
  #define sample8to16(a) (short)(((int)(a)-0x80)<<8)

  /* Locks/Unlocks the buffer mutex */
  #define LOCK_BUFFERS(ga)    kn_mtx_lock(&ga->buffer_mutex)
  #define UNLOCK_BUFFERS(ga)  kn_mtx_unlock(&ga->buffer_mutex)
  #define DELETE_MUTEX(ga)    kn_mtx_destroy(&ga->buffer_mutex)
  #define EXIT_THREAD(ga)     pthread_exit(0)

  /* Locks/Unlocks the buffer mutex if not paused */
  #define LOCK_BUFFERS_NP(ga)    if (!audioPaused(ga)) LOCK_BUFFERS(ga)
  #define UNLOCK_BUFFERS_NP(ga)  if (!audioPaused(ga)) UNLOCK_BUFFERS(ga)

  /* play the audio buffer- dummy arg needed by some compilers */
  #define SEND_BUFFER_TO_DEVICE(a) write(ga->audio_fd, a, abufsize )

  /* what makes us sleep */
  #define SLEEP_TYPE unsigned long long
  #define SLEEP_CALL(a) usleep(a)

   /* Default OSS device to use */
  #define DEFAULT_DEVICE_NAME "/device/dsfx"

  #define DEFAULT_NUM_AUDIO_BUFFERS 4

     struct ElatePlatformData {
           void *appname;
           void *dsfxinst;
           void *dsfxhandle;
           int aio_counter;
           ELATE_AIO *aio_blockP;
     };


#endif  /* Elate - AMigaDE_Native */


#ifdef __SYMBIAN32__

  #include <stdlib.h>
  #include <string.h>

  #include "symbian.h"

  #define sample16to8(a) (unsigned char) ((a>>8)+0x80);
  #define sample8to16(a) (short)(((int)(a)-0x80)<<8)

  /* Locks/Unlocks the buffer mutex */
  #define LOCK_BUFFERS(ga)    MutexWait(ga->buffer_mutex)
  #define UNLOCK_BUFFERS(ga)  MutexSignal(ga->buffer_mutex)
  #define DELETE_MUTEX(ga)    MutexClose(ga->buffer_mutex)
  #define EXIT_THREAD(ga)     /*ThreadExit(0)*/

  /* Locks/Unlocks the buffer mutex if not paused */
  #define LOCK_BUFFERS_NP(ga)    if (!audioPaused(ga)) LOCK_BUFFERS(ga)
  #define UNLOCK_BUFFERS_NP(ga)  if (!audioPaused(ga)) UNLOCK_BUFFERS(ga)

  /* play the audio buffer- dummy arg needed by some compilers */
  #define SEND_BUFFER_TO_DEVICE(a) /*WriteToAudioBuffer(ga->audio_fd, a, abufsize)*/

  /* what makes us sleep */
  #define SLEEP_TYPE int
  #define SLEEP_CALL(a) symbian_usleep(a)

  #define DEFAULT_NUM_AUDIO_BUFFERS 4

  /* Until we fix background */
  #define popen(a,b)
  #define pclose(a)
  #define fread(a,b,c,d) 0

#endif  /* Symbian32 */


#define AUDIO_BUFFER_SIZE ga->audioBufferSize

#ifdef HAVE_REGISTER_MODIFIER
	#define _REGISTER register
#else
	#define _REGISTER
#endif

#endif /* internals_h */
