
#include "sssd.h"
#include "internals.h"

/* Generic register function - note 'mode' contains flags */

int sssd_registerSample( struct sssdaudio* ga, void* sampledata, int channels, int freq, int bits, int samples, int mode) 
{

    struct ga_sample* sample;
    int id;
    int i,byps;
    unsigned char *cursample;

    if( ! audioValid( ga ) )
        return( ERR_NOT_INITIALIZED );

    /* Check entry count */
    if( ga->num_samples >= ga->max_samples )
    {
        #ifndef __NOPRINTING__
        fprintf( stderr, "sssd: sample list is full\n" );
        #endif
        ga->lastERR=ERR_SAMPLES_FULL;
        return( ERR_SAMPLES_FULL );
    }

    id     = ga->num_samples;
    sample = &ga->samples[ id ];

    ga->num_samples++;

        
    /* Print info */
    #ifndef __NOPRINTING__
    fprintf (stderr, "Registered Sample: %d samples, %d channels, %d rate\n",(int) samples, channels, freq);
    #endif    

    /* Lock buffers while loading sample */
    LOCK_BUFFERS(ga);

    /* Setup sample length */
    sample->numsamples = samples;

    /* Create buffer */
    sample->data = sampledata;

    /* Blank one or the other channel depending on mode */

    if(ga->channels!=1) {
      if (mode == COPY_LEFT) {
    	 cursample=(unsigned char *)sample->data;
    	 if (ga->byps==2) {
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 2;
    	  }
    	 } else if (ga->byps==4) {
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 1;
    	   *cursample=0;
    	   cursample += 3;
    	  }
    	 }
      } else if (mode == COPY_RIGHT) {
    	 cursample=(unsigned char *)sample->data;
    	 if (ga->byps==2) {
    	 cursample++;
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 2;
    	  }
    	 } else if (ga->byps==4) {
     	  cursample += 2;
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 1;
    	   *cursample=0;
    	   cursample += 3;
    	  }
    	 }
      }
    }

    /* set any sample flags */
    sample->flags=gaIS_DYNAMIC;


    byps=ga->byps;

    if(ga->channels!=channels) {
         sample->flags|=gaCHAN_CONVERT;
         if(ga->channels==2) byps>>=1;
         else
         if(ga->channels==1) byps<<=1;
    }
    
    #ifndef __NOPRINTING__
       fprintf (stderr, "Channel Convert: %d\n",(sample->flags & gaCHAN_CONVERT ? 1 : 0 ));
    #endif    

    /* Calculate end-of-sample position pointer */ 
    

    sample->dataendmarker = (char *)sample->data + (byps * samples);


    UNLOCK_BUFFERS(ga);

    ga->lastERR=0;
    return( id );
}

