
/*

    PLACE THIS APP, HELLO.WAV. AND SSSD.DLL INTO YOUR DEVICE'S
    \TEMP DIRECTORY

*/


// wince_hello.cpp : Defines the entry point for the application.
//

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers

// Windows Header Files:
#include <windows.h>

#ifndef _WIN32_WCE
#define _WIN32_WCE
#endif

#include "../loadSimpleWave.h"  /* routine to load simple Wav file */
#include "../../src/sssd.h"

FILE *OutFile;

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{

    /* libsssd handle */
    struct sssdaudio *ga;

    /* sample references */
    int hello_id=0;
    
    /* play references (not needed) */
    int hello_play;
   
    /* wave file info */
    waveFile *waveData;
    
	/* output log */
	OutFile=fopen("\\Temp\\wince_hello.log","wa");

    /* Initialize audio */
    ga=sssd_initAudio(
    		 1,		/* Channels: 1=mono, 2=stereo */
    		 22050,		/* Frequency */
		 8,		/* bits per sample (8|16) */
		 1,		/* Maximum simul voices */
		 1,		/* Maximum samples */
		 0,              /* Size of audio hardware bufffer 0=let system decide */
		 DEFAULT_NUMBUFFERS, /* Number of audio hardware bufffers */
		 0);		/* Flags */

    /* open soundcard device */
    if( sssd_isError(sssd_startAudio( ga, DEFAULT_DEVICE)) ) exit(-1);

    /* load simple Wav file */
    waveData=loadSimpleWave("\\Temp\\hello.wav");
    if(waveData==NULL) { 
       fprintf(OutFile,"Could not load hello.wav\n");
       exit(-1);
    }
    
    /* pre-cache samples ... keeping id's */
    /* simpleWave2copySample is a macro that fills in args to sssd_copySample */
    /* when used with loadSimpleWave() */
    hello_id   = simpleWave2copySample( ga, waveData, COPY_DONT_RESAMPLE);
    
    /* As soon as sample is copy cached, we can delete the local copy */
    free(waveData);
    
    /* make sure there wasn't an error copying the sample */
    if(sssd_isError(hello_id)) {
       fprintf(OutFile,"Error copying sample to sssd (%d)\n",hello_id);
       exit(-2);
    }

    /* play sample */
    hello_play=sssd_playSample( ga, hello_id, 1, 256, 0 );
    
    /* Wait until the sample has finished playing */
    sssd_blockUntilComplete(ga, hello_play);

    /* Shut audio down */
    sssd_stopAudio( ga ); 
    sssd_freeAudio( ga );
    
    fclose(OutFile);

	return 0;
}

waveFile *loadSimpleWave(char *filename) {


   FILE *fp;
   void *data;
   int fileSize;
   waveFile *waveData;
   int i1, *ip1;
   
   fp=fopen(filename,"rb");
   if(fp==NULL) return NULL;

   fprintf(OutFile,"loadSimpleWave: %s opened\n",filename);
   
   fseek(fp,0,SEEK_END);
   fileSize=ftell(fp);
   fseek(fp,0,SEEK_SET); /* no 'rewind' in WinCE */

   fprintf(OutFile,"loadSimpleWave: filesize=%d\n",fileSize);
   
   data=malloc(fileSize);
   if(data==NULL) {
     fclose(fp);
     return NULL;
   }
   
   fprintf(OutFile,"loadSimpleWave: %d bytes allocated\n",fileSize);
 
   fread(data,fileSize,1,fp);
   fclose(fp);

   fprintf(OutFile,"loadSimpleWave: file %s read into memory \n",filename);
 
   waveData=(waveFile *)data;
   
   fprintf(OutFile,"loadSimpleWave: file info...\n\t"
	    "RIFF   =0x%x\n\t"
            "chunkID=0x%x\n\t"
	    "format =%d\n\t"
	    "chnls  =%d\n\t"
	    "freq   =%d\n\t"
	    "bits   =%d\n\t",
	    waveData->RIFF_dsize,
	    waveData->chunkID,
	    waveData->format,
	    waveData->channels,
	    waveData->freq,
	    waveData->bits);
 
   if(waveData->chunkID!=0x20746d66) {
     free(data);
     return NULL;
   }
   
   if(waveData->format!=1) {
     free(data);
     return NULL;
   }
   
   fputs("loadSimpleWave: Finding data chunk",OutFile);
   
   /* find data chunk*/
   i1=waveData->chunkSize+20;  /* the next format */
   ip1=(int *)((char *)data+i1);
   
   while(*ip1!=0x61746164 && i1<fileSize-16) {
     fprintf(OutFile,"loadSimpleWave: Skipping chunk id 0x%x (0x%x bytes)\n",*ip1,ip1[1]);
     i1+=(ip1[1]+8);
     ip1=(int *)((char *)data+i1);     
   }
   
   /* did we not find a data chunk? */
   if(i1>=fileSize+16) {
     free(data);
     return NULL;
   }
   
   waveData->PCMdata=&ip1[2];
   waveData->RIFF_dsize=ip1[1];
   waveData->chunkSize=waveData->RIFF_dsize/waveData->align;

   fprintf(OutFile,"loadSimpleWave: PCM data found: %d bytes/%d samples (0x%x)\n",waveData->RIFF_dsize,waveData->chunkSize,*(int*)waveData->PCMdata);

return waveData;
   
}
